/*!
* Module dependencies.
*/

var mongoose = require('mongoose');
var config = require('../../config/env/development');
var async = require('async');
var utils = require('../../utils');
var fs = require('fs');
var path = require('path');

var Building = mongoose.model('Building');


/**
* @api {get} /buildings GET all Buildings
* @apiName GET buildings
* @apiGroup Buildings
*
*
* @apiSuccess {Number} appeal  Appeal
* @apiSuccess {Number} unlockLevel  Level at which the building unlocks
* @apiSuccess {Number} cost  Cost to place the building
* @apiSuccess {String} imagePath  url of the image
* @apiSuccess {Number} subType  building subtype
* @apiSuccess {String} type  building type
* @apiSuccess {String} name  Name of the building
*/

exports.read = function (req, res) {
    var query = {};
    if (req.query.type) {
        query['type'] = req.query.type;
    }
    if (req.query.enabled) {
        query['enabled'] = req.query.enabled;
    }
    var language = req.query.language;
    Building.find(query).sort({ unlockLevel: 1 }).exec(async function (err, buildings) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read building data', JSON.stringify(err.errors));
        } else {
            if(language){
               let response=[], data;;
                for (let index = 0; index < buildings.length; index++) {
                    data=await transalationService(buildings[index],language);
                    response.push(data.building)
                    if (data.update)
                      udpate(buildings[index]._id, data.building)
                }
                return utils.sendSuccessResponse(res, 200, response);
            }else{
                return utils.sendSuccessResponse(res, 200, buildings);
            }
            
        }
    })
}

/**
* @api {get} /buildings/:buildingId GET Building detail
* @apiName GET buildings/:buildingId
* @apiGroup Buildings
*
* @apiParam Buildings {String} building unique ID
*
* @apiSuccess {Number} appeal  Appeal
* @apiSuccess {Number} unlockLevel  Level at which the building unlocks
* @apiSuccess {Number} cost  Cost to place the building
* @apiSuccess {String} imagePath  url of the image
* @apiSuccess {Number} subType  building subtype
* @apiSuccess {String} type  building type
* @apiSuccess {String} name  Name of the building
*/

exports.detail = function (req, res) {
    var buildingId = req.params.buildingId;
    var language = req.query.language;
    var data;
    Building.findOne({ _id: buildingId }).lean().exec(async function (err, building) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read building data', JSON.stringify(err.errors));
        } else {
            if(language){
                data=await transalationService(building,language);
                if (data.update)
                  udpate(buildingId, data.building)
            }
            return utils.sendSuccessResponse(res, 200,language? data.building:building);
          }
    })
}

/**
* @api {post} /buildings POST Create a new building
* @apiName POST buildings
* @apiGroup Buildings
*
*
* @apiSuccess {Number} appeal  Appeal
* @apiSuccess {Number} unlockLevel  Level at which the building unlocks
* @apiSuccess {Number} cost  Cost to place the building
* @apiSuccess {String} imagePath  url of the image
* @apiSuccess {Number} subType  building subtype
* @apiSuccess {String} type  building type
* @apiSuccess {String} name  Name of the building
*/
exports.create =  function (req, res) {
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    req.body.upgradeInfo = JSON.parse(req.body.upgradeInfo);
    var building = new Building(req.body);
    building.save(function (err, savedBuilding) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to create a new building', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, savedBuilding);
        }
    });
}


/**
* @api {put} /buildings/:buildingId UPDATE building details
* @apiName UPDATE buildings/:buildingId
* @apiGroup Buildings
*
* @apiParam Buildings {String} building unique ID
*
* @apiSuccess {Number} appeal  Appeal
* @apiSuccess {Number} unlockLevel  Level at which the building unlocks
* @apiSuccess {Number} cost  Cost to place the building
* @apiSuccess {String} imagePath  url of the image
* @apiSuccess {Number} subType  building subtype
* @apiSuccess {String} type  building type
* @apiSuccess {String} name  Name of the building
*/
exports.update = function (req, res) {
    var buildingId = req.params.buildingId;
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    var updateParams = req.body;
    updateParams.translation={name:{},dialogues:{}};
    Building.findOneAndUpdate({ _id: buildingId }, { $set: updateParams }, { new: true, upsert: true }).exec(function (err, updatedBuilding) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to update building', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, updatedBuilding);
        }
    })
}

/**
* @api {delete} /buildings/:buildingId DELETE a building
* @apiName DELETE buildings/:buildingId
* @apiGroup Buildings
*
* @apiParam Buildings {String} building unique ID
*
* @apiSuccess {Number} appeal  Appeal
* @apiSuccess {Number} unlockLevel  Level at which the building unlocks
* @apiSuccess {Number} cost  Cost to place the building
* @apiSuccess {String} imagePath  url of the image
* @apiSuccess {Number} subType  building subtype
* @apiSuccess {String} type  building type
* @apiSuccess {String} name  Name of the building
*/
exports.delete = function (req, res) {
    var buildingId = req.params.buildingId;
    Building.remove({ _id: buildingId }).exec(function (err, data) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to delete building entry', JSON.stringify(err.errors));
        } else {
            if (data['result']['n'] == 1) {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "Deleted building",
                        success: true
                    }
                );
            } else {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "No building found to delete",
                        success: false
                    }
                );
            }
        }
    })
}

exports.createConfig = function (req, res) {
    var buildingType = req.query.type;
    var configLink = "";
    async.waterfall([
        function (callback) {
            Building.find({ type: buildingType }).sort({ id: -1 }).exec(callback);
        },
        function (buildings, callback) {
            var stringifiedJSON = JSON.stringify(buildings);
            var fileName = utils.randomNameGenerator() + '.json';
            var savePath = config.assets.path + '/' + fileName;
            configLink = config.assets.url + '/' + fileName;
            fs.writeFile(savePath, stringifiedJSON, callback);
        }
    ], function (err, results) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read building config', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, { link: configLink });
        }
    });

}

function udpate(id,data){
    Building.findOneAndUpdate({ _id: id }, { $set: data }, { new: false, upsert: false }).exec(function (err, updatedBuilding) {
        if (err) {
            console.log(err);
        }
    })
}

async function transalationService(building,language){
    var response={
        translation:{
            name:{[language]:{}},
            dialogues:{[language]:[]}
    }};
   
    var update=false;
    if(!building.translation){
        building["translation"]=response.translation
    }
           
    if (!building || !building.translation || !building.translation.name || !building.translation.name[language] || !building.translation.name[language].length) {
      update = true;
      response.translation.name[language] = building.name? await utils.translateText(building.name, language):"";
      building["translation"]["name"] =(building["translation"]&&building["translation"]["name"])? {
        ...building["translation"]["name"],
        ...response.translation.name
      }:response.translation.name
    }
    if (!building || !building.translation || !building.translation.dialogues || !building.translation.dialogues[language] || !building.translation.dialogues[language].length) {
      update = true;
      for (let index = 0; index < building.dialogues.length; index++) {
        response.translation.dialogues[language][index] = building.dialogues[index]?await utils.translateText(building.dialogues[index], language):"";
      }
      building["translation"]["dialogues"] =(building["translation"] && building["translation"]["dialogues"])? {
        ...building["translation"]["dialogues"],
        ...response.translation.dialogues
      }:response.translation.dialogues
    }

    return {building,update};
}